function allMosaic=makeMosaic(all_4D,gapWidth,gapColor,normalizationType,figTitle)
%                                                                          (ver 2.12, May 15, 2016)
% generate figure of mosaic of blur kernels that compose the fovation operator, as well as other mosaics.
%
%
% REFERENCES:
% [1] A. Foi and G. Boracchi, "Foveated self-similarity in nonlocal image filtering",
%     Proc. IS&T/SPIE EI2012 - Human Vision and Electronic Imaging XVII, 8291-32,
%     Burlingame (CA), USA, Jan. 2012.  http://doi.org/10.1117/12.912217
% [2] A. Foi and G. Boracchi, "Anisotropically Foveated Nonlocal Image Denoising",
%     Proc. IEEE Int. Conf. Image Process. (ICIP 2013), pp. 464-468, Melbourne, Australia, Sep. 2013.
%     http://doi.org/10.1109/ICIP.2013.6738096
% [3] A. Foi and G. Boracchi, "Foveated nonlocal self-similarity", to appear in Int. J. Computer Vision, 2016.
%     http://doi.org/10.1007/s11263-016-0898-1
% 
%  All available online at http://www.cs.tut.fi/~foi/
%
%
%  Author:  Alessandro Foi, Tampere University of Technology, Finland
%           All rights reserved.
% --------------------------------------------------------------------------------------
%

if ~exist('gapWidth','var')
    gapWidth=1;
end
if ~exist('gapColor','var')
    gapColor=0.8;
end

if ~exist('normalizationType','var')
    normalizationType=0;
end

size_U_1=size(all_4D,3);
size_U_2=size(all_4D,4);
sizeTile_1=size(all_4D,1); % this is typically size_U_1+2*extraPad;
sizeTile_2=size(all_4D,2); % this is typically size_U_2+2*extraPad;

allMosaic=zeros( size_U_1*(sizeTile_1+gapWidth)-gapWidth,size_U_2*(sizeTile_2+gapWidth)-gapWidth)+gapColor;

maxAbsTiles=max(max(abs(all_4D),[],1),[],2);
maxAbsAllTiles=max(maxAbsTiles(:));
% sumAbsTiles=sum(sum(abs(all_4D),1),2);
% maxSumAbsTiles=max(sumAbsTiles(:));


for jj1=1:size_U_1;
    for jj2=1:size_U_2;
        tile=squeeze(all_4D(:,:,jj1,jj2));
        if normalizationType==0
            tile=tile/maxAbsTiles(1,1,jj1,jj2);
        elseif normalizationType==1
            tile=tile/(maxAbsTiles(1,1,jj1,jj2)*2)+0.5;
        elseif normalizationType==2
            tile=tile/(maxAbsAllTiles*2)+0.5;
        end
        allMosaic((sizeTile_1+gapWidth)*(jj1-1)+1:(sizeTile_1+gapWidth)*(jj1-1)+sizeTile_1,(sizeTile_2+gapWidth)*(jj2-1)+1:(sizeTile_2+gapWidth)*(jj2-1)+sizeTile_2)=tile;
    end
end
figHandle=figure;
axisHandle=axes('parent',figHandle);
image(allMosaic*64,'parent',axisHandle,'CDataMapping','direct'), colormap(figHandle,gray);
% % % % image(allMosaic,'parent',axisHandle,'CDataMapping','scaled'), colormap(figHandle,gray);

axis(axisHandle,'equal','tight','off');
if exist('figTitle','var')
    title(axisHandle,figTitle);
end
truesize(figHandle);
set(figHandle,'color',max(0,min(1,interp1(linspace(min(allMosaic(:)),max(allMosaic(:)),size(get(figHandle,'Colormap'),1)),get(figHandle,'Colormap'),gapColor,'linear','extrap'))));
pause(0.001);
